/******************************************************************************
 * JibsConfiguration.java - Parse the jIBS configuration.
 * $Id: JibsConfiguration.java,v 1.7 2011/06/03 17:06:28 dick Exp $
 * 
 * jIBS - A Java clone of FIBS
 * Copyright 2002 - Axel Leucht
 * Copyright 2011 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: JibsConfiguration.java,v $
 * Revision 1.7  2011/06/03 17:06:28  dick
 * If the user data area doesn't exist, then create and populate it.
 *
 * Revision 1.6  2011/06/01 01:33:15  dick
 * Javadoc.
 *
 * Revision 1.5  2011/06/01 01:32:07  dick
 * Javadoc.
 *
 * Revision 1.4  2011/06/01 01:22:54  dick
 * Read and write the window position parameters from a separate properties file.
 * The practice is that the primary jibs.properties is "read-only" as it user config and well commented.
 * jibsWindow.properties is more dynamic in that its values may change per session (and it may not exist).
 *
 * Revision 1.3  2011/05/31 14:50:49  dick
 * Bucko headers.
 *
 * Revision 1.2  2011/05/29 05:18:17  dick
 * Read the config from the resources, then the local user's file.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is jIBS, <https://sourceforge.net/projects/fibs/>.
 * The Initial Developer of the Original Code is Axel Leucht.
 * 
 */
package net.sourceforge.jibs.server;

import java.io.BufferedOutputStream;
import java.io.BufferedReader;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;
import java.util.StringTokenizer;

import org.apache.log4j.Logger;

/** Parse the jIBS configuration.
 * Configuration is loaded from the Resources and then from 
 * the user area, and then from any file specified. 
 *
 * @author Axel Leucht
 * @author Dick Balaska
 * @version $Revision: 1.7 $ <br> $Date: 2011/06/03 17:06:28 $
 * @see <a href="http://cvs.buckosoft.com/Projects/jIBS/jIBS/src/main/java/net/sourceforge/jibs/server/JibsConfiguration.java">cvs JibsConfiguration.java</a>
 */
public class JibsConfiguration {
	private static Logger logger = Logger.getLogger(JibsConfiguration.class);

	/** The resources subdirectory to find config stuff */
	public final static String confDir = "conf/";


	private static final String confName = "jibs.properties";

	private JibsServer jibsServer = null;
	private Map<String, String> allParameters = null;

	/** Constructor for JibsConfiguration  <br>
	 * Read the properties, in this order, from:
	 * <ol>
	 * <li>The jar resource <i>resource_root</i>/conf/jibs.properties
	 * <li>The user data area <i>${user.home}</i>/jibs/jibs.properties
	 * <li>Any filename passed as an argument on the command line.
	 * </ol>
	 * @param jibsServer A reference to The jibsServer
	 * @param fileName Optional custom properties file to load after the system defaults.
	 */
	public JibsConfiguration (JibsServer jibsServer, String fileName ) {
		this.jibsServer = jibsServer;
		this.allParameters = new HashMap<String, String>(  );
		try {
			InputStream is = ClassLoader.getSystemResourceAsStream(confDir + confName);
			readOneFile(new BufferedReader(new InputStreamReader(is)));

			String s = System.getProperty("user.home");
			if (s != null) {
				File f = new File(s + File.separator + "jibs" + File.separator + confName);
				if (f.canRead()) {
					logger.info( "Reading configuration file (" + f.getAbsolutePath() + ")" );
					Reader         reader = new FileReader(f);
					readOneFile(new BufferedReader( reader ));

				}
			}
			if (fileName != null && fileName.length() > 0) {
				File f = new File(fileName);
				if (f.canRead()) {
					logger.info( "Reading configuration file (" + fileName + ")" );
					Reader         reader = new FileReader( new File( fileName ) );
					readOneFile(new BufferedReader( reader ));
				}
			}
			if (Boolean.parseBoolean(this.getResource("useSwing"))) {
				try {
					Reader reader = new FileReader(new File(getWindowPositionFileName()));
					readOneFile(new BufferedReader(reader));
				} catch (Exception e) {}
			}
			
		} catch ( Exception e ) {
			logger.warn( "JibsConfiguration", e );
		}
	}

	/** Fetch a String from our config properties.
	 * @param key The name of the property to fetch.
	 * @return The String from the map, or if not found then log an error and return null. 
	 */
	public String getResource ( String key ) {
		String value = allParameters.get( key );

		if ( value == null ) {
			logger.error( "Parameter " + key + " not found in configuration" );
		}

		return value;
	}

	/** Return a resource as an integer.
	 * @param key The name of the property to fetch.
	 * @return An int value of the resource, or 0 if not found.
	 */
	public int	getIntResource(String key) {
		String s = allParameters.get(key);
		if (s == null)
			return(0);
		return(Integer.parseInt(s));
	}

	/** Return whether we have data for this key
	 * @param key The key to check
	 * @return true if this key is in the map
	 */
	public boolean hasProperty(String key) {
		return(this.allParameters.containsKey(key));
	}

	/** Read one file's worth of properties, parse, and stuff them in the map.
	 * @param buffer The file to read
	 */
	private void readOneFile(BufferedReader buffer) {
		String line;

		try {
			while ( ( line = buffer.readLine(  ) ) != null ) {
				if ( !line.startsWith( "#" ) ) {
					StringTokenizer stoken = new StringTokenizer( line, "=" );

					while ( stoken.hasMoreTokens(  ) ) {
						String par   = stoken.nextToken(  );
						String value = "";

						if ( stoken.hasMoreTokens(  ) ) {
							value = stoken.nextToken(  );
						}

						allParameters.put( par, value );
						logger.debug( par + "=" + value );
					}
				}
			}
		} catch ( Exception e ) {
			logger.warn( "JibsConfiguration", e );
			jibsServer.logException(e);
		}

	}

	/** Given a string, substitute any System properties in that string.
	 * A property is denoted with {}, i.e. <code>{user.home}</code>
	 * @param in The string to unwind
	 * @return The resultant string.  The original string is returned if no left brace '{' is found.
	 */
	public static String unwindProperties(String in) {
		int i = in.indexOf('{');
		if (i == -1)
			return(in);
		int j = in.indexOf('}');
		String out = in.substring(0, i);
		String s = System.getProperty(in.substring(i+1, j));
		out += s;
		out += in.substring(j+1);
		return(out);

	}
	
	private String getWindowPositionFileName() {
		return(getUserDataDirectory() + File.separator + "jibsWindow.properties");
	}

	/** Return the location for user data
	 * @return The absolute string for {user.home}/jIBS
	 */
	public static String getUserDataDirectory() {
		String s = "{user.home}" + File.separator + "jIBS";
		return(JibsConfiguration.unwindProperties(s));
	}
	
	/** Return the location of our hsqldb database
	 * @return The absolute string for {user.home}/jIBS/database
	 */
	public static String getUserDatabaseDirectory() {
		return(getUserDataDirectory() + File.separator + "database");
	}

	/** Write the window position out to the window properties file
	 * @param top The y of the main window
	 * @param left The x of the main window
	 * @param width The width of the main window
	 * @param height The height of the main window
	 */
	public void writeWindowPosition(int top, int left, int width, int height) {
		Properties p = new Properties();
		p.put("mainTop", "" + top);
		p.put("mainLeft", "" + left);
		p.put("mainWidth", "" + width);
		p.put("mainHeight", "" + height);
		try {
			File f = new File(getWindowPositionFileName());
			FileOutputStream fos = new FileOutputStream(f);
			p.store(fos, "jIBS window properties");
			fos.close();
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}
		
	}

	/** If the user data area doesn't exist, then create and populate it
	 * @throws Exception Die if something goes bad here.
	 */
	public static void maybeInitializeJibsFiles() throws Exception {
		File f = new File(getUserDatabaseDirectory());
		if (!f.isDirectory()) {
			System.out.println("jIBS: Creating user data area...");	// no logger yet
			System.out.println("jIBS: data home = '" + getUserDataDirectory() + "'");
			if (!f.mkdirs()) {
				System.err.println("Failed to create dir '" + f.getAbsolutePath() + "'");
				System.err.println("Aborting.");
				System.exit(101);
			}
			InputStream is = ClassLoader.getSystemResourceAsStream(confDir + confName);
			f = new File(getUserDataDirectory() + File.separator + confName);
			copyFile(is, f);
			is = ClassLoader.getSystemResourceAsStream(
					"database" + File.separator + "hsqldb" + File.separator + "empty" 
					+ File.separator + "jibs.script");
			f = new File(getUserDatabaseDirectory() + File.separator + "jibs.script");
			copyFile(is, f);
		}
	}

	/**
	 * Copies a file to another file.
	 * @param is Input stream to copy from 
	 * @param target File to copy to. 
	 * @exception Exception Thrown if a problem is encountered. 
	 */  
	private static void copyFile(InputStream is, File target) throws Exception {
		DataInputStream i = null;
		DataOutputStream o = null;
		try {
			i = new DataInputStream(is);
			o = new DataOutputStream(
					new BufferedOutputStream(
							new FileOutputStream(target)));
			int buflen = 1024;
			byte[] buffer  = new byte[buflen];
			while ((buflen = i.read(buffer)) != -1)
				o.write(buffer, 0, buflen);
			o.flush();
		}
		catch (IOException e) {
			throw new Exception("Problem copying file", e);
		}
		catch (NullPointerException e) {
			throw new Exception("Null file cannot be copied.");
		}
		finally {
			try {
				i.close();
			}
			catch (IOException e) {}
			catch (NullPointerException e) {}
			try {
				o.close();
			}
			catch (IOException e) {}
			catch (NullPointerException e) {}
		}
	}
}
